@@ -1,5 +1,3 @@
-// -*- C++ -*-
-
 /* main.cc
  *
  * Copyright (C) 2002 The libxml++ development team
@@ -23,9 +21,11 @@
 #include <config.h>
 #endif
 
+#include "../testutilities.h"
 #include <libxml++/libxml++.h>
 #include <iostream>
-#include <stdlib.h>
+#include <fstream>
+#include <cstdlib>
 
 void print_node(const xmlpp::Node* node, bool substitute_entities, unsigned int indentation = 0)
 {  
@@ -35,40 +35,39 @@ void print_node(const xmlpp::Node* node, bool substitute_entities, unsigned int
   if (substitute_entities)
   {
     // Entities have been substituted. Print the text nodes.
-    const xmlpp::TextNode* nodeText = dynamic_cast<const xmlpp::TextNode*>(node);
+    const auto nodeText = dynamic_cast<const xmlpp::TextNode*>(node);
     if (nodeText && !nodeText->is_white_space())
     {
-      std::cout << indent << "text = " << nodeText->get_content() << std::endl;
+      std::cout << indent << "text = " << CatchConvertError(nodeText->get_content()) << std::endl;
     }
   }
   else
   {
     // Entities have not been substituted. Print the entity reference nodes.
-    const xmlpp::EntityReference* nodeEntityReference = dynamic_cast<const xmlpp::EntityReference*>(node);
+    const auto nodeEntityReference = dynamic_cast<const xmlpp::EntityReference*>(node);
     if (nodeEntityReference)
     {
-      std::cout << indent << "entity reference name = " << nodeEntityReference->get_name() << std::endl;
-      std::cout << indent <<  "  resolved text = " << nodeEntityReference->get_resolved_text() << std::endl;
-      std::cout << indent <<  "  original text = " << nodeEntityReference->get_original_text() << std::endl;
+      std::cout << indent << "entity reference name = " << CatchConvertError(nodeEntityReference->get_name()) << std::endl;
+      std::cout << indent << "  resolved text = " << CatchConvertError(nodeEntityReference->get_resolved_text()) << std::endl;
+      std::cout << indent << "  original text = " << CatchConvertError(nodeEntityReference->get_original_text()) << std::endl;
     }
   } // end if (substitute_entities)
 
-  const xmlpp::ContentNode* nodeContent = dynamic_cast<const xmlpp::ContentNode*>(node);
+  const auto nodeContent = dynamic_cast<const xmlpp::ContentNode*>(node);
   if(!nodeContent)
   {
     //Recurse through child nodes:
-    xmlpp::Node::NodeList list = node->get_children();
-    for(xmlpp::Node::NodeList::iterator iter = list.begin(); iter != list.end(); ++iter)
+    for(const auto& child : node->get_children())
     {   
-      print_node(*iter, substitute_entities, indentation + 2); //recursive
+      print_node(child, substitute_entities, indentation + 2); //recursive
     }
   }
 }
 
 int main(int argc, char* argv[])
 {
-  // Set the global C++ locale to the user-configured locale,
-  // so we can use std::cout with UTF-8, via Glib::ustring, without exceptions.
+  // Set the global C++ locale to the user-specified locale. Then we can
+  // hopefully use std::cout with UTF-8, via Glib::ustring, without exceptions.
   std::locale::global(std::locale(""));
 
   std::string filepath;
@@ -92,11 +91,21 @@ int main(int argc, char* argv[])
       xmlpp::DomParser parser;
       parser.set_validate();
       parser.set_substitute_entities(substitute_entities);
-      parser.parse_file(filepath);
+
+      // Two ways of reading the XML file.
+      if (substitute_entities)
+        parser.parse_file(filepath);
+      else
+      {
+        std::ifstream instream(filepath.c_str());
+        if (!instream)
+          throw xmlpp::internal_error("Could not open file " + filepath);
+        instream >> parser;
+      }
       if(parser)
       {
         //Walk the tree:
-        const xmlpp::Node* pNode = parser.get_document()->get_root_node(); //deleted by DomParser.
+        const auto pNode = parser.get_document()->get_root_node(); //deleted by DomParser.
         print_node(pNode, substitute_entities);
       }
     }
@@ -113,4 +122,3 @@ int main(int argc, char* argv[])
 
   return return_code;
 }
-
