@@ -31,7 +31,7 @@ xmlXPathObject* eval_common(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathContext* ctxt = xmlXPathNewContext(node->doc);
+  auto ctxt = xmlXPathNewContext(node->doc);
   if (!ctxt)
     throw xmlpp::internal_error("Could not create XPath context for " + xpath);
   ctxt->node = node;
@@ -45,7 +45,7 @@ xmlXPathObject* eval_common(const Glib::ustring& xpath,
         reinterpret_cast<const xmlChar*>(it->second.c_str()));
   }
 
-  xmlXPathObject* xpath_value = xmlXPathEvalExpression(
+  auto xpath_value = xmlXPathEvalExpression(
     reinterpret_cast<const xmlChar*>(xpath.c_str()), ctxt);
 
   xmlXPathFreeContext(ctxt);
@@ -77,7 +77,7 @@ bool eval_common_to_boolean(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathObject* xpath_value = eval_common(xpath, namespaces, result_type, node);
+  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
   const int result = xmlXPathCastToBoolean(xpath_value);
   xmlXPathFreeObject(xpath_value);
   return static_cast<bool>(result);
@@ -88,7 +88,7 @@ double eval_common_to_number(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathObject* xpath_value = eval_common(xpath, namespaces, result_type, node);
+  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
   const double result = xmlXPathCastToNumber(xpath_value);
   xmlXPathFreeObject(xpath_value);
   return result;
@@ -99,7 +99,7 @@ Glib::ustring eval_common_to_string(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
   xmlpp::XPathResultType* result_type, xmlNode* node)
 {
-  xmlXPathObject* xpath_value = eval_common(xpath, namespaces, result_type, node);
+  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
   xmlChar* result = xmlXPathCastToString(xpath_value);
   xmlXPathFreeObject(xpath_value);
   if (result)
@@ -111,6 +111,18 @@ Glib::ustring eval_common_to_string(const Glib::ustring& xpath,
   return Glib::ustring();
 }
 
+// Common part of all add_child*() methods.
+xmlpp::Element* add_child_common(const Glib::ustring& name, xmlNode* child, xmlNode* node)
+{
+  if (!node)
+  {
+    xmlFreeNode(child);
+    throw xmlpp::internal_error("Could not add child element node " + name);
+  }
+  xmlpp::Node::create_wrapper(node);
+  return static_cast<xmlpp::Element*>(node->_private);
+}
+
 } // anonymous namespace
 
 namespace xmlpp
@@ -119,7 +131,10 @@ namespace xmlpp
 Node::Node(xmlNode* node)
   : impl_(node)
 {
-   impl_->_private = this;
+  if (!impl_)
+    throw internal_error("xmlNode pointer cannot be nullptr");
+
+  impl_->_private = this;
 }
 
 Node::~Node()
@@ -133,7 +148,7 @@ const Element* Node::get_parent() const
 Element* Node::get_parent()
 {
   if(!(cobj()->parent && cobj()->parent->type == XML_ELEMENT_NODE))
-    return 0;
+    return nullptr;
 
   Node::create_wrapper(cobj()->parent);
   return static_cast<Element*>(cobj()->parent->_private);
@@ -147,7 +162,7 @@ const Node* Node::get_next_sibling() const
 Node* Node::get_next_sibling()
 {
   if(!cobj()->next)
-    return 0;
+    return nullptr;
 
   Node::create_wrapper(cobj()->next);
   return static_cast<Node*>(cobj()->next->_private);
@@ -161,7 +176,7 @@ const Node* Node::get_previous_sibling() const
 Node* Node::get_previous_sibling()
 {
   if(!cobj()->prev)
-    return 0;
+    return nullptr;
 
   Node::create_wrapper(cobj()->prev);
   return static_cast<Node*>(cobj()->prev->_private);
@@ -169,7 +184,7 @@ Node* Node::get_previous_sibling()
 
 static Node* _convert_node(xmlNode* node)
 {
-  Node* res = 0;
+  Node* res = nullptr;
   if(node)
   {
     Node::create_wrapper(node);
@@ -180,9 +195,9 @@ static Node* _convert_node(xmlNode* node)
 
 Node* Node::get_first_child(const Glib::ustring& name)
 {
-  xmlNode* child = impl_->children;
+  auto child = impl_->children;
   if(!child)
-    return 0;
+    return nullptr;
 
   do
   {
@@ -191,7 +206,7 @@ Node* Node::get_first_child(const Glib::ustring& name)
   }
   while((child = child->next));
    
-  return 0;
+  return nullptr;
 }
 
 const Node* Node::get_first_child(const Glib::ustring& name) const
@@ -201,7 +216,7 @@ const Node* Node::get_first_child(const Glib::ustring& name) const
 
 Node::NodeList Node::get_children(const Glib::ustring& name)
 {
-   xmlNode* child = impl_->children;
+   auto child = impl_->children;
    if(!child)
      return NodeList();
 
@@ -224,56 +239,70 @@ const Node::NodeList Node::get_children(const Glib::ustring& name) const
 Element* Node::add_child(const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  _xmlNode* child = create_new_child_node(name, ns_prefix);
-  _xmlNode* node = xmlAddChild(impl_, child);
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not add child element node " + name);
-  }
-  Node::create_wrapper(node);
-  return static_cast<Element*>(node->_private);
+  auto child = create_new_child_node(name, ns_prefix);
+  auto node = xmlAddChild(impl_, child);
+  return add_child_common(name, child, node);
 }
 
 Element* Node::add_child(xmlpp::Node* previous_sibling, 
                          const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  if(!previous_sibling)
-    return 0;
+  if (!previous_sibling)
+    return nullptr;
 
-  _xmlNode* child = create_new_child_node(name, ns_prefix);
-  _xmlNode* node = xmlAddNextSibling(previous_sibling->cobj(), child);
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not add child element node " + name);
-  }
-  Node::create_wrapper(node);
-  return static_cast<Element*>(node->_private);
+  auto child = create_new_child_node(name, ns_prefix);
+  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
+  return add_child_common(name, child, node);
 }
 
 Element* Node::add_child_before(xmlpp::Node* next_sibling, 
                          const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  if(!next_sibling)
-    return 0;
+  if (!next_sibling)
+    return nullptr;
 
-  _xmlNode* child = create_new_child_node(name, ns_prefix);
-  _xmlNode* node = xmlAddPrevSibling(next_sibling->cobj(), child);
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not add child element node " + name);
-  }
-  Node::create_wrapper(node);
-  return static_cast<Element*>(node->_private);
+  auto child = create_new_child_node(name, ns_prefix);
+  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
+  return add_child_common(name, child, node);
+}
+
+Element* Node::add_child_with_new_ns(const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
+  auto node = xmlAddChild(impl_, child);
+  return add_child_common(name, child, node);
+}
+
+Element* Node::add_child_with_new_ns(xmlpp::Node* previous_sibling,
+  const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  if (!previous_sibling)
+    return nullptr;
+
+  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
+  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
+  return add_child_common(name, child, node);
+}
+
+Element* Node::add_child_before_with_new_ns(xmlpp::Node* next_sibling,
+  const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  if (!next_sibling)
+    return nullptr;
+
+  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
+  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
+  return add_child_common(name, child, node);
 }
 
 _xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ustring& ns_prefix)
 {
-   xmlNs* ns = 0;
+   xmlNs* ns = nullptr;
 
    if(impl_->type != XML_ELEMENT_NODE)
    {
@@ -283,7 +312,7 @@ _xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ust
    if(ns_prefix.empty())
    {
      //Retrieve default namespace if it exists
-     ns = xmlSearchNs(impl_->doc, impl_, 0);
+     ns = xmlSearchNs(impl_->doc, impl_, nullptr);
    }
    else
    {
@@ -298,6 +327,36 @@ _xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ust
    return xmlNewNode(ns, (const xmlChar*)name.c_str());
 }
 
+_xmlNode* Node::create_new_child_node_with_new_ns(const Glib::ustring& name,
+  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
+{
+  if (impl_->type != XML_ELEMENT_NODE)
+    throw internal_error("You can only add child nodes to element nodes.");
+
+  auto child = xmlNewNode(nullptr, (const xmlChar*)name.c_str());
+  if (!child)
+    throw internal_error("Could not create new element node.");
+
+  auto ns = xmlNewNs(child, (const xmlChar*)(ns_uri.empty() ? nullptr : ns_uri.c_str()),
+                       (const xmlChar*)(ns_prefix.empty() ? nullptr : ns_prefix.c_str()) );
+  // xmlNewNs() does not create a namespace node for the predefined xml prefix.
+  // It's usually defined in the document and not in any specific node.
+  if (!ns && ns_prefix == "xml")
+  {
+    ns = xmlSearchNs(impl_->doc, impl_, (const xmlChar*)ns_prefix.c_str());
+    if (ns && (ns_uri != (ns->href ? (const char*)ns->href : "")))
+      ns = nullptr;
+  }
+  if (!ns)
+  {
+    xmlFreeNode(child);
+    throw internal_error("Could not create new namespace node.");
+  }
+
+  xmlSetNs(child, ns);
+
+  return child;
+}
 
 void Node::remove_child(Node* node)
 {
@@ -305,7 +364,7 @@ void Node::remove_child(Node* node)
   //This would require a more complex memory management API.
   if (!node)
     return;
-  xmlNode* cnode = node->cobj();
+  auto cnode = node->cobj();
   Node::free_wrappers(cnode); //This delete the C++ node (not this) itself.
   xmlUnlinkNode(cnode);
   xmlFreeNode(cnode);
@@ -314,10 +373,10 @@ void Node::remove_child(Node* node)
 Node* Node::import_node(const Node* node, bool recursive)
 {
   if (!node)
-    return 0;
+    return nullptr;
 
   //Create the node, by copying:
-  xmlNode* imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_->doc, recursive);
+  auto imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_->doc, recursive);
   if (!imported_node)
   {
     throw exception("Unable to copy the node that shall be imported");
@@ -325,8 +384,8 @@ Node* Node::import_node(const Node* node, bool recursive)
 
   if (imported_node->type == XML_ATTRIBUTE_NODE && impl_->type == XML_ELEMENT_NODE)
   {
-    xmlAttr* old_attr = xmlHasNsProp(impl_, imported_node->name,
-      imported_node->ns ? imported_node->ns->href : 0);
+    auto old_attr = xmlHasNsProp(impl_, imported_node->name,
+      imported_node->ns ? imported_node->ns->href : nullptr);
     if (old_attr && old_attr->type != XML_ATTRIBUTE_DECL)
     {
       // *this has an attribute with the same name as the imported attribute.
@@ -337,7 +396,7 @@ Node* Node::import_node(const Node* node, bool recursive)
   }
 
   //Add the node:
-  xmlNode* added_node = xmlAddChild(this->cobj(), imported_node);
+  auto added_node = xmlAddChild(this->cobj(), imported_node);
 
   if (!added_node)
   {
@@ -390,7 +449,7 @@ Glib::ustring Node::get_path() const
 
 static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
 {
-  xmlXPathObject* result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
+  auto result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
 
   if(!result)
   {
@@ -407,7 +466,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
     throw internal_error("Only nodeset result types are supported.");
   }
 
-  xmlNodeSet* nodeset = result->nodesetval;
+  auto nodeset = result->nodesetval;
   NodeSet nodes;
   if( nodeset && !xmlXPathNodeSetIsEmpty(nodeset))
   {
@@ -415,7 +474,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
     nodes.reserve(count);
     for (int i = 0; i != count; ++i)
     {
-      xmlNode* cnode = xmlXPathNodeSetItem(nodeset, i);
+      auto cnode = xmlXPathNodeSetItem(nodeset, i);
       if(!cnode)
       {
         std::cerr << "Node::find_impl: The xmlNode was null." << std::endl;
@@ -433,7 +492,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
       //TODO: Check for other cnode->type values?
   
       Node::create_wrapper(cnode);
-      Node* cppNode = static_cast<Node*>(cnode->_private);
+      auto cppNode = static_cast<Node*>(cnode->_private);
       nodes.push_back(cppNode);
     }
   }
@@ -450,7 +509,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
 
 NodeSet Node::find(const Glib::ustring& xpath) const
 {
-  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
+  auto ctxt = xmlXPathNewContext(impl_->doc);
   if (!ctxt)
     throw internal_error("Could not create XPath context for " + xpath);
   ctxt->node = impl_;
@@ -461,7 +520,7 @@ NodeSet Node::find(const Glib::ustring& xpath) const
 NodeSet Node::find(const Glib::ustring& xpath,
 		   const PrefixNsMap& namespaces) const
 {
-  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
+  auto ctxt = xmlXPathNewContext(impl_->doc);
   if (!ctxt)
     throw internal_error("Could not create XPath context for " + xpath);
   ctxt->node = impl_;
@@ -477,7 +536,7 @@ NodeSet Node::find(const Glib::ustring& xpath,
 
 bool Node::eval_to_boolean(const Glib::ustring& xpath, XPathResultType* result_type) const
 {
-  return eval_common_to_boolean(xpath, 0, result_type, impl_);
+  return eval_common_to_boolean(xpath, nullptr, result_type, impl_);
 }
 
 bool Node::eval_to_boolean(const Glib::ustring& xpath, const PrefixNsMap& namespaces,
@@ -488,7 +547,7 @@ bool Node::eval_to_boolean(const Glib::ustring& xpath, const PrefixNsMap& namesp
 
 double Node::eval_to_number(const Glib::ustring& xpath, XPathResultType* result_type) const
 {
-  return eval_common_to_number(xpath, 0, result_type, impl_);
+  return eval_common_to_number(xpath, nullptr, result_type, impl_);
 }
 
 double Node::eval_to_number(const Glib::ustring& xpath, const PrefixNsMap& namespaces,
@@ -499,7 +558,7 @@ double Node::eval_to_number(const Glib::ustring& xpath, const PrefixNsMap& names
 
 Glib::ustring Node::eval_to_string(const Glib::ustring& xpath, XPathResultType* result_type) const
 {
-  return eval_common_to_string(xpath, 0, result_type, impl_);
+  return eval_common_to_string(xpath, nullptr, result_type, impl_);
 }
 
 Glib::ustring Node::eval_to_string(const Glib::ustring& xpath, const PrefixNsMap& namespaces,
@@ -527,8 +586,7 @@ Glib::ustring Node::get_namespace_prefix() const
     const xmlAttribute* const attr = reinterpret_cast<const xmlAttribute*>(impl_);
     return attr->prefix ? (const char*)attr->prefix : "";
   }
-
-  if(impl_ && impl_->ns && impl_->ns->prefix)
+  else if(impl_->ns && impl_->ns->prefix)
     return (char*)impl_->ns->prefix;
   else
     return Glib::ustring();
@@ -550,7 +608,7 @@ Glib::ustring Node::get_namespace_uri() const
     return Glib::ustring();
   }
 
-  if(impl_ && impl_->ns && impl_->ns->href)
+  if(impl_->ns && impl_->ns->href)
     return (char*)impl_->ns->href;
   else
     return Glib::ustring();
@@ -564,7 +622,7 @@ void Node::set_namespace(const Glib::ustring& ns_prefix)
   }
 
   //Look for the existing namespace to use:
-  xmlNs* ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)(ns_prefix.empty() ? 0 : ns_prefix.c_str()) );
+  auto ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)(ns_prefix.empty() ? nullptr : ns_prefix.c_str()) );
   if(ns)
   {
       //Use it for this element:
@@ -580,8 +638,8 @@ void Node::create_wrapper(xmlNode* node)
 {
   if(node->_private)
   {
-	  //Node already wrapped, skip
-	  return;
+    //Node already wrapped, skip
+    return;
   }
 
   switch (node->type)
@@ -681,7 +739,7 @@ void Node::free_wrappers(xmlNode* node)
   if (node->type != XML_ENTITY_REF_NODE)
   {
     //Walk the children list.
-    for (xmlNode* child = node->children; child; child = child->next)
+    for (auto child = node->children; child; child = child->next)
       free_wrappers(child);
   }
 
@@ -691,21 +749,21 @@ void Node::free_wrappers(xmlNode* node)
     //Node types that have no properties
     case XML_DTD_NODE:
       delete static_cast<Dtd*>(node->_private);
-      node->_private = 0;
+      node->_private = nullptr;
       return;
     case XML_ATTRIBUTE_NODE:
     case XML_ELEMENT_DECL:
     case XML_ATTRIBUTE_DECL:
     case XML_ENTITY_DECL:
       delete static_cast<Node*>(node->_private);
-      node->_private = 0;
+      node->_private = nullptr;
       return;
     case XML_DOCUMENT_NODE:
       //Do not free now. The Document is usually the one who owns the caller.
       return;
     default:
       delete static_cast<Node*>(node->_private);
-      node->_private = 0;
+      node->_private = nullptr;
       break;
   }
 
@@ -714,7 +772,7 @@ void Node::free_wrappers(xmlNode* node)
   //_xmlNode::properties would be a nonsense value, leading to crashes,
   //(and shown as valgrind warnings), so we return above, to avoid 
   //checking it here.
-  for(xmlAttr* attr = node->properties; attr; attr = attr->next)
+  for(auto attr = node->properties; attr; attr = attr->next)
     free_wrappers(reinterpret_cast<xmlNode*>(attr));
 }
 
