@@ -8,11 +8,12 @@
 // because it temporarily undefines G_DISABLE_DEPRECATED while it includes glib.h.
 #include <glibmm/threads.h> // For Glib::Threads::Mutex. Needed until the next API/ABI break.
 
+#include "libxml++/exceptions/wrapped_exception.h"
 #include "libxml++/parsers/parser.h"
 
 #include <libxml/parser.h>
 
-#include <memory> //For auto_ptr.
+#include <memory> //For unique_ptr.
 #include <map>
 
 //TODO: See several TODOs in parser.h for changes at the next API/ABI break.
@@ -61,7 +62,7 @@ void on_parser_warning(const xmlpp::Parser* parser, const Glib::ustring& message
 namespace xmlpp {
 
 Parser::Parser()
-: context_(0), exception_(0), validate_(false), substitute_entities_(false) //See doxygen comment on set_substiute_entities().
+: context_(nullptr), exception_(nullptr), validate_(false), substitute_entities_(false) //See doxygen comment on set_substiute_entities().
 {
 
 }
@@ -145,7 +146,7 @@ void Parser::initialize_context()
 
   // Take a copy of the extra data, so we don't have to access
   // the extra_parser_data map more than necessary.
-  const ExtraParserData extra_parser_data_this = extra_parser_data[this];
+  const auto extra_parser_data_this = extra_parser_data[this];
   lock.release();
 
   //Disactivate any non-standards-compliant libxml1 features.
@@ -201,15 +202,15 @@ void Parser::release_underlying()
 {
   if(context_)
   {
-    context_->_private = 0; //Not really necessary.
+    context_->_private = nullptr; //Not really necessary.
     
-    if( context_->myDoc != 0 )
+    if( context_->myDoc != nullptr )
     {
       xmlFreeDoc(context_->myDoc);
     }
 
     xmlFreeParserCtxt(context_);
-    context_ = 0;
+    context_ = nullptr;
   }
 }
 
@@ -307,13 +308,13 @@ void Parser::callback_error_or_warning(MsgType msg_type, void* ctx,
 {
   //See xmlHTMLValidityError() in xmllint.c in libxml for more about this:
   
-  xmlParserCtxtPtr context = (xmlParserCtxtPtr)ctx;
+  auto context = (xmlParserCtxtPtr)ctx;
   if(context)
   {
-    Parser* parser = static_cast<Parser*>(context->_private);
+    auto parser = static_cast<Parser*>(context->_private);
     if(parser)
     {
-      Glib::ustring ubuff = format_xml_error(&context->lastError);
+      auto ubuff = format_xml_error(&context->lastError);
       if (ubuff.empty())
       {
         // Usually the result of formatting var_args with the format string msg
@@ -346,9 +347,9 @@ void Parser::callback_error_or_warning(MsgType msg_type, void* ctx,
             break;
         }
       }
-      catch(const exception& e)
+      catch (...)
       {
-        parser->handleException(e);
+        parser->handle_exception();
       }
     }
   }
@@ -365,14 +366,50 @@ void Parser::handleException(const exception& e)
   //release_underlying();
 }
 
+void Parser::handle_exception()
+{
+  delete exception_;
+  exception_ = nullptr;
+
+  try
+  {
+    throw; // Rethrow current exception
+  }
+  catch (const exception& e)
+  {
+    exception_ = e.Clone();
+  }
+#ifdef LIBXMLXX_HAVE_EXCEPTION_PTR
+  catch (...)
+  {
+    exception_ = new wrapped_exception(std::current_exception());
+  }
+#else
+  catch (const std::exception& e)
+  {
+    exception_ = new exception(e.what());
+  }
+  catch (...)
+  {
+    exception_ = new exception("An exception was thrown that is not derived from std::exception or xmlpp::exception.\n"
+      "It could not be caught and rethrown because this platform does not support std::exception_ptr.");
+  }
+#endif
+
+  if (context_)
+    xmlStopParser(context_);
+
+  //release_underlying();
+}
+
 void Parser::check_for_exception()
 {
   check_for_validity_messages();
   
   if(exception_)
   {
-    std::auto_ptr<exception> tmp ( exception_ );
-    exception_ = 0;
+    std::unique_ptr<exception> tmp(exception_);
+    exception_ = nullptr;
     tmp->Raise();
   }
 }
